@@ -5,7 +5,6 @@
 #include "util/stringUtils.h"
 
 #define FF_LOCALIP_DISPLAY_NAME "Local IP"
-#define FF_LOCALIP_NUM_FORMAT_ARGS 7
 #pragma GCC diagnostic ignored "-Wsign-conversion"
 
 static int sortIps(const FFLocalIpResult* left, const FFLocalIpResult* right)
@@ -25,7 +24,7 @@ static void formatKey(const FFLocalIpOptions* options, FFLocalIpResult* ip, uint
     else
     {
         ffStrbufClear(key);
-        FF_PARSE_FORMAT_STRING_CHECKED(key, &options->moduleArgs.key, 4, ((FFformatarg[]){
+        FF_PARSE_FORMAT_STRING_CHECKED(key, &options->moduleArgs.key, ((FFformatarg[]) {
             FF_FORMAT_ARG(index, "index"),
             FF_FORMAT_ARG(ip->name, "name"),
             FF_FORMAT_ARG(ip->mac, "mac"),
@@ -77,6 +76,12 @@ static void printIp(FFLocalIpResult* ip, bool markDefaultRoute)
         putchar(']');
         flag = true;
     }
+    if (ip->flags.length) {
+        if (flag) fputs(" <", stdout);
+        ffStrbufWriteTo(&ip->flags, stdout);
+        putchar('>');
+        flag = true;
+    }
     if (markDefaultRoute && flag && ip->defaultRoute)
         fputs(" *", stdout);
 }
@@ -143,7 +148,7 @@ void ffPrintLocalIp(FFLocalIpOptions* options)
                     else
                         ffStrbufSetF(&speedStr, "%u Mbps", (unsigned) ip->speed);
                 }
-                FF_PRINT_FORMAT_CHECKED(key.chars, 0, &options->moduleArgs, FF_PRINT_TYPE_NO_CUSTOM_KEY, FF_LOCALIP_NUM_FORMAT_ARGS, ((FFformatarg[]){
+                FF_PRINT_FORMAT_CHECKED(key.chars, 0, &options->moduleArgs, FF_PRINT_TYPE_NO_CUSTOM_KEY, ((FFformatarg[]){
                     FF_FORMAT_ARG(ip->ipv4, "ipv4"),
                     FF_FORMAT_ARG(ip->ipv6, "ipv6"),
                     FF_FORMAT_ARG(ip->mac, "mac"),
@@ -151,6 +156,7 @@ void ffPrintLocalIp(FFLocalIpOptions* options)
                     FF_FORMAT_ARG(ip->defaultRoute, "is-default-route"),
                     FF_FORMAT_ARG(ip->mtu, "mtu"),
                     FF_FORMAT_ARG(speedStr, "speed"),
+                    FF_FORMAT_ARG(ip->flags, "flags"),
                 }));
             }
             ++index;
@@ -163,6 +169,7 @@ void ffPrintLocalIp(FFLocalIpOptions* options)
         ffStrbufDestroy(&ip->ipv4);
         ffStrbufDestroy(&ip->ipv6);
         ffStrbufDestroy(&ip->mac);
+        ffStrbufDestroy(&ip->flags);
     }
 }
 
@@ -236,6 +243,15 @@ bool ffParseLocalIpCommandOptions(FFLocalIpOptions* options, const char* key, co
         return true;
     }
 
+    if (ffStrEqualsIgnCase(subKey, "show-flags"))
+    {
+        if (ffOptionParseBoolean(value))
+            options->showType |= FF_LOCALIP_TYPE_FLAGS_BIT;
+        else
+            options->showType &= ~FF_LOCALIP_TYPE_FLAGS_BIT;
+        return true;
+    }
+
     if(ffStrEqualsIgnCase(subKey, "compact"))
     {
         if (ffOptionParseBoolean(value))
@@ -348,6 +364,15 @@ void ffParseLocalIpJsonObject(FFLocalIpOptions* options, yyjson_val* module)
             continue;
         }
 
+        if (ffStrEqualsIgnCase(key, "showFlags"))
+        {
+            if (yyjson_get_bool(val))
+                options->showType |= FF_LOCALIP_TYPE_FLAGS_BIT;
+            else
+                options->showType &= ~FF_LOCALIP_TYPE_FLAGS_BIT;
+            continue;
+        }
+
         if (ffStrEqualsIgnCase(key, "compact"))
         {
             if (yyjson_get_bool(val))
@@ -415,6 +440,9 @@ void ffGenerateLocalIpJsonConfig(FFLocalIpOptions* options, yyjson_mut_doc* doc,
         if (options->showType & FF_LOCALIP_TYPE_SPEED_BIT)
             yyjson_mut_obj_add_bool(doc, module, "showSpeed", true);
 
+        if (options->showType & FF_LOCALIP_TYPE_FLAGS_BIT)
+            yyjson_mut_obj_add_bool(doc, module, "showFlags", true);
+
         if (options->showType & FF_LOCALIP_TYPE_COMPACT_BIT)
             yyjson_mut_obj_add_bool(doc, module, "compact", true);
 
@@ -445,13 +473,20 @@ void ffGenerateLocalIpJsonResult(FF_MAYBE_UNUSED FFLocalIpOptions* options, yyjs
     FF_LIST_FOR_EACH(FFLocalIpResult, ip, results)
     {
         yyjson_mut_val* obj = yyjson_mut_arr_add_obj(doc, arr);
-        yyjson_mut_obj_add_bool(doc, obj, "defaultRoute", ip->defaultRoute);
-        yyjson_mut_obj_add_strbuf(doc, obj, "ipv4", &ip->ipv4);
-        yyjson_mut_obj_add_strbuf(doc, obj, "ipv6", &ip->ipv6);
-        yyjson_mut_obj_add_strbuf(doc, obj, "mac", &ip->mac);
         yyjson_mut_obj_add_strbuf(doc, obj, "name", &ip->name);
-        yyjson_mut_obj_add_int(doc, obj, "mtu", ip->mtu);
-        yyjson_mut_obj_add_int(doc, obj, "speed", ip->speed);
+        yyjson_mut_obj_add_bool(doc, obj, "defaultRoute", ip->defaultRoute);
+        if (options->showType & FF_LOCALIP_TYPE_IPV4_BIT)
+            yyjson_mut_obj_add_strbuf(doc, obj, "ipv4", &ip->ipv4);
+        if (options->showType & FF_LOCALIP_TYPE_IPV6_BIT)
+            yyjson_mut_obj_add_strbuf(doc, obj, "ipv6", &ip->ipv6);
+        if (options->showType & FF_LOCALIP_TYPE_MAC_BIT)
+            yyjson_mut_obj_add_strbuf(doc, obj, "mac", &ip->mac);
+        if (options->showType & FF_LOCALIP_TYPE_MTU_BIT)
+            yyjson_mut_obj_add_int(doc, obj, "mtu", ip->mtu);
+        if (options->showType & FF_LOCALIP_TYPE_SPEED_BIT)
+            yyjson_mut_obj_add_int(doc, obj, "speed", ip->speed);
+        if (options->showType & FF_LOCALIP_TYPE_FLAGS_BIT)
+            yyjson_mut_obj_add_strbuf(doc, obj, "flags", &ip->flags);
     }
 
     FF_LIST_FOR_EACH(FFLocalIpResult, ip, results)
@@ -460,39 +495,37 @@ void ffGenerateLocalIpJsonResult(FF_MAYBE_UNUSED FFLocalIpOptions* options, yyjs
         ffStrbufDestroy(&ip->ipv4);
         ffStrbufDestroy(&ip->ipv6);
         ffStrbufDestroy(&ip->mac);
+        ffStrbufDestroy(&ip->flags);
     }
 }
 
-void ffPrintLocalIpHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_LOCALIP_MODULE_NAME, "{1}", FF_LOCALIP_NUM_FORMAT_ARGS, ((const char* []) {
-        "Local IPv4 address - ipv4",
-        "Local IPv6 address - ipv6",
-        "Physical (MAC) address - mac",
-        "Interface name - ifname",
-        "Is default route - is-default-route",
-        "MTU size in bytes - mtu",
-        "Link speed (formatted) - speed",
-    }));
-}
+static FFModuleBaseInfo ffModuleInfo = {
+    .name = FF_LOCALIP_MODULE_NAME,
+    .description = "List local IP addresses (v4 or v6), MAC addresses, etc",
+    .parseCommandOptions = (void*) ffParseLocalIpCommandOptions,
+    .parseJsonObject = (void*) ffParseLocalIpJsonObject,
+    .printModule = (void*) ffPrintLocalIp,
+    .generateJsonResult = (void*) ffGenerateLocalIpJsonResult,
+    .generateJsonConfig = (void*) ffGenerateLocalIpJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"IPv4 address", "ipv4"},
+        {"IPv6 address", "ipv6"},
+        {"MAC address", "mac"},
+        {"Interface name", "ifname"},
+        {"Is default route", "is-default-route"},
+        {"MTU size in bytes", "mtu"},
+        {"Link speed (formatted)", "speed"},
+        {"Interface flags", "flags"},
+    }))
+};
 
 void ffInitLocalIpOptions(FFLocalIpOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_LOCALIP_MODULE_NAME,
-        "List local IP addresses (v4 or v6), MAC addresses, etc",
-        ffParseLocalIpCommandOptions,
-        ffParseLocalIpJsonObject,
-        ffPrintLocalIp,
-        ffGenerateLocalIpJsonResult,
-        ffPrintLocalIpHelpFormat,
-        ffGenerateLocalIpJsonConfig
-    );
+    options->moduleInfo = ffModuleInfo;
     ffOptionInitModuleArg(&options->moduleArgs, "󰩟");
 
     options->showType = FF_LOCALIP_TYPE_IPV4_BIT | FF_LOCALIP_TYPE_PREFIX_LEN_BIT
-        #ifndef __ANDROID__
+        #if !__ANDROID__ /*Permission denied*/ && !__OpenBSD__ /*Report invalid argument for some reason*/ && !__DragonFly__ /*Doesn't work*/
             | FF_LOCALIP_TYPE_DEFAULT_ROUTE_ONLY_BIT
         #endif
     ;
